@@ -12,10 +12,11 @@ use options::{Command, Options};
 
 use structopt::StructOpt;
 
-use crate::meilisearch::Meilisearch;
+use crate::{meilisearch::Meilisearch, options::IndexesCommand};
 
 type DocId = u32;
 type UpdateId = u32;
+type TaskId = u32;
 type DumpId = String;
 
 fn main() -> Result<()> {
@@ -56,8 +57,22 @@ fn main() -> Result<()> {
                 ids => meili.delete_batch(ids)?,
             }
         }
-        Command::Search { search_terms } => meili.search(search_terms.join(" "))?,
+        Command::Search {
+            search_terms,
+            interactive: false,
+        } => meili.search(search_terms.join(" "))?,
+        Command::Search {
+            search_terms,
+            interactive: true,
+        } => meili.interactive_search(search_terms.join(" "))?,
         Command::Settings { r#async } => meili.r#async(r#async).settings()?,
+        Command::Index { command } => match command {
+            IndexesCommand::All => meili.get_all_indexes()?,
+            IndexesCommand::Get { index } => meili.get_index(index)?,
+            IndexesCommand::Create { index, primary } => meili.create_index(index, primary)?,
+            IndexesCommand::Update { index, primary } => meili.update_index(index, primary)?,
+            IndexesCommand::Delete { index } => meili.delete_index(index)?,
+        },
         Command::Dump {
             r#async,
             dump_id: None,
@@ -70,6 +85,7 @@ fn main() -> Result<()> {
         Command::Version => meili.version()?,
         Command::Stats => meili.stats()?,
         Command::Status { update_id, watch } => meili.r#async(!watch).status(update_id)?,
+        Command::Task { task_id, watch } => meili.r#async(!watch).task(task_id)?,
     }
 
     Ok(())
