@@ -5,14 +5,15 @@ use std::{
 };
 
 use crate::{
-    format::{write_json, write_response_full},
-    DocId, DumpId, Options, UpdateId,
+    format::{write_json, write_response_full, write_response_headers},
+    DocId, DumpId, Options, TaskId, UpdateId,
 };
 use anyhow::Result;
 use indicatif::ProgressBar;
 use reqwest::{
     blocking::{Client, RequestBuilder, Response},
-    header::USER_AGENT,
+    header::{AUTHORIZATION, USER_AGENT},
+    StatusCode,
 };
 use serde_json::{json, Map, Value};
 
@@ -24,6 +25,7 @@ pub struct Meilisearch {
     pub interval: usize,
     pub r#async: bool,
     pub user_agent: String,
+    pub verbose: usize,
 }
 
 impl From<&Options> for Meilisearch {
@@ -38,6 +40,7 @@ impl From<&Options> for Meilisearch {
                 .user_agent
                 .clone()
                 .unwrap_or_else(|| format!("mieli/{}", env!("CARGO_PKG_VERSION"))),
+            verbose: options.verbose,
         }
     }
 }
@@ -56,7 +59,7 @@ impl Meilisearch {
     }
 
     pub fn put(&self, url: impl AsRef<str>) -> RequestBuilder {
-        self.request(|c| c.get(url.as_ref()))
+        self.request(|c| c.put(url.as_ref()))
     }
 
     pub fn delete(&self, url: impl AsRef<str>) -> RequestBuilder {
@@ -71,6 +74,7 @@ impl Meilisearch {
         if let Some(ref key) = self.key {
             req_builder
                 .header("X-Meili-API-Key", key)
+                .header(AUTHORIZATION, &format!("Bearer {}", key))
                 .header(USER_AGENT, &self.user_agent)
         } else {
             req_builder.header(USER_AGENT, &self.user_agent)
@@ -84,8 +88,7 @@ impl Meilisearch {
                 self.addr, self.index, docid
             ))
             .send()?;
-        self.handle_response(response)?;
-        Ok(())
+        self.handle_response(response)
     }
 
     pub fn get_all_documents(&self) -> Result<()> {
@@ -93,8 +96,7 @@ impl Meilisearch {
         let response = self
             .get(&format!("{}/indexes/{}/documents", self.addr, self.index))
             .send()?;
-        self.handle_response(response)?;
-        Ok(())
+        self.handle_response(response)
     }
 
     pub fn index_documents(
@@ -128,16 +130,14 @@ impl Meilisearch {
                     .send()?
             }
         };
-        self.handle_response(response)?;
-        Ok(())
+        self.handle_response(response)
     }
 
     pub fn delete_all(&self) -> Result<()> {
         let response = self
             .delete(format!("{}/indexes/{}/documents", self.addr, self.index))
             .send()?;
-        self.handle_response(response)?;
-        Ok(())
+        self.handle_response(response)
     }
 
     pub fn delete_one(&self, docid: DocId) -> Result<()> {
@@ -147,8 +147,7 @@ impl Meilisearch {
                 self.addr, self.index, docid
             ))
             .send()?;
-        self.handle_response(response)?;
-        Ok(())
+        self.handle_response(response)
     }
 
     pub fn delete_batch(&self, docids: &[DocId]) -> Result<()> {
@@ -159,8 +158,7 @@ impl Meilisearch {
             ))
             .json(docids)
             .send()?;
-        self.handle_response(response)?;
-        Ok(())
+        self.handle_response(response)
     }
 
     pub fn search(&self, search: String) -> Result<()> {
@@ -172,79 +170,154 @@ impl Meilisearch {
         if !search.is_empty() {
             value.insert("q".to_string(), json!(search));
         }
-
-        if atty::is(atty::Stream::Stdout) {
-            self.interactive_search(search, value)?;
-        } else {
-            let response = self
-                .post(format!("{}/indexes/{}/search", self.addr, self.index))
-                .header("Content-Type", "application/json")
-                .json(&value)
-                .send()?;
-
-            self.handle_response(response)?;
-        }
-        Ok(())
-    }
-
-    pub fn settings(&self) -> Result<()> {
-        let mut buffer = Vec::new();
-        stdin().read_to_end(&mut buffer);
-
         let response = self
-            .post(format!("{}/indexes/{}/settings", self.addr, self.index))
+            .post(format!("{}/indexes/{}/search", self.addr, self.index))
             .header("Content-Type", "application/json")
-            .body(buffer)
+            .json(&value)
             .send()?;
 
         self.handle_response(response)
     }
 
-    pub fn status(&self, uid: UpdateId) -> Result<()> {
+    pub fn interactive_search(&self, search: String) -> Result<()> {
+        if atty::isnt(atty::Stream::Stdout) {
+            return self.search(search);
+        }
+
+        let mut value: Map<String, Value> = if atty::isnt(atty::Stream::Stdin) {
+            serde_json::from_reader(stdin())?
+        } else {
+            Map::new()
+        };
+        if !search.is_empty() {
+            value.insert("q".to_string(), json!(search));
+        }
+
+        self.run_interactive_search(search, value)
+    }
+
+    pub fn settings(&self) -> Result<()> {
+        let response = if atty::is(atty::Stream::Stdin) {
+            self.get(format!("{}/indexes/{}/settings", self.addr, self.index))
+                .send()?
+        } else {
+            let mut buffer = Vec::new();
+            stdin().read_to_end(&mut buffer);
+
+            self.post(format!("{}/indexes/{}/settings", self.addr, self.index))
+                .header("Content-Type", "application/json")
+                .body(buffer)
+                .send()?
+        };
+
+        self.handle_response(response)
+    }
+
+    pub fn get_all_indexes(&self) -> Result<()> {
+        let response = self.get(format!("{}/indexes", self.addr)).send()?;
+        self.handle_response(response)
+    }
+
+    pub fn get_index(&self, index: Option<String>) -> Result<()> {
+        let index = index.unwrap_or(self.index.to_string());
+        let response = self
+            .get(format!("{}/indexes/{}", self.addr, index))
+            .send()?;
+        self.handle_response(response)
+    }
+
+    pub fn create_index(&self, index: Option<String>, primary_key: Option<String>) -> Result<()> {
+        let index = index.unwrap_or(self.index.to_string());
+        let mut body = json!({ "uid": index });
+        if let Some(primary_key) = primary_key {
+            body["primaryKey"] = json!(primary_key);
+        }
+        let response = self
+            .post(format!("{}/indexes", self.addr))
+            .json(&body)
+            .send()?;
+        self.handle_response(response)
+    }
+
+    pub fn update_index(&self, index: Option<String>, primary_key: Option<String>) -> Result<()> {
+        let index = index.unwrap_or(self.index.to_string());
+        let mut body = json!({});
+        dbg!(&primary_key);
+        if let Some(primary_key) = primary_key {
+            body["primaryKey"] = json!(primary_key);
+        }
+        dbg!(&body);
+        let response = self
+            .put(format!("{}/indexes/{}", self.addr, index))
+            .json(&body)
+            .send()?;
+        self.handle_response(response)
+    }
+
+    pub fn delete_index(&self, index: Option<String>) -> Result<()> {
+        let index = index.unwrap_or(self.index.to_string());
+        let response = self
+            .delete(format!("{}/indexes/{}", self.addr, index))
+            .send()?;
+        self.handle_response(response)
+    }
+
+    pub fn status(&self, uid: Option<UpdateId>) -> Result<()> {
         let response = self
             .get(format!(
                 "{}/indexes/{}/updates/{}",
-                self.addr, self.index, uid
+                self.addr,
+                self.index,
+                uid.map_or("".to_string(), |uid| uid.to_string())
             ))
             .send()?;
-        self.handle_response(response)?;
-        Ok(())
+        self.handle_response(response)
+    }
+
+    pub fn task(&self, tid: Option<TaskId>) -> Result<()> {
+        let response = self
+            .get(format!(
+                "{}/indexes/{}/tasks/{}",
+                self.addr,
+                self.index,
+                tid.map_or("".to_string(), |uid| uid.to_string())
+            ))
+            .send()?;
+        self.handle_response(response)
     }
 
     pub fn create_dump(&self) -> Result<()> {
         let response = self.post(format!("{}/dumps", self.addr)).send()?;
-        self.handle_response(response)?;
-        Ok(())
+        self.handle_response(response)
     }
 
     pub fn dump_status(&self, dump_id: DumpId) -> Result<()> {
         let response = self
             .get(format!("{}/dumps/{}/status", self.addr, dump_id))
             .send()?;
-        self.handle_response(response)?;
-        Ok(())
+        self.handle_response(response)
     }
 
     pub fn healthcheck(&self) -> Result<()> {
         let response = self.get(format!("{}/health", self.addr)).send()?;
-        self.handle_response(response)?;
-        Ok(())
+        self.handle_response(response)
     }
 
     pub fn version(&self) -> Result<()> {
         let response = self.get(format!("{}/version", self.addr)).send()?;
-        self.handle_response(response)?;
-        Ok(())
+        self.handle_response(response)
     }
 
     pub fn stats(&self) -> Result<()> {
         let response = self.get(format!("{}/stats", self.addr)).send()?;
-        self.handle_response(response)?;
-        Ok(())
+        self.handle_response(response)
     }
 
     pub fn handle_response(&self, response: Response) -> Result<()> {
-        let response = write_response_full(response)?;
+        if response.status() == StatusCode::NO_CONTENT {
+            return write_response_headers(&response, self.verbose);
+        }
+        let response = write_response_full(response, self.verbose)?;
         if self.r#async {
             return Ok(());
         }
@@ -253,7 +326,24 @@ impl Meilisearch {
 
         let buffer = String::new();
 
-        if let Some(uid) = response["updateId"].as_i64() {
+        if let Some(uid) = response["uid"].as_i64() {
+            loop {
+                let response = self.get(format!("{}/tasks/{}", self.addr, uid)).send()?;
+                let json = response.json::<Value>()?;
+                match json["status"].as_str() {
+                    None => {
+                        return Ok(());
+                    }
+                    Some(msg @ "succeeded") | Some(msg @ "failed") => {
+                        spinner.finish_with_message(msg.to_string());
+                        write_json(json);
+                        break;
+                    }
+                    Some(status) => spinner.set_message(status.to_string()),
+                }
+                std::thread::sleep(std::time::Duration::from_millis(self.interval as u64));
+            }
+        } else if let Some(uid) = response["updateId"].as_i64() {
             loop {
                 let response = self
                     .get(format!(
